/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __reExport = (target, mod, secondTarget) => (__copyProps(target, mod, "default"), secondTarget && __copyProps(secondTarget, mod, "default"));
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// src/ImageRegistry.js
var ImageRegistry_exports = {};
__export(ImageRegistry_exports, {
  default: () => ImageRegistry_default
});
module.exports = __toCommonJS(ImageRegistry_exports);

// esm-externals:@splunk/ui-utils/i18n
var i18n_exports = {};
__export(i18n_exports, {
  default: () => i18n_default
});
var defaultImport = __toESM(require("@splunk/ui-utils/i18n"));
__reExport(i18n_exports, require("@splunk/ui-utils/i18n"));
var i18n_default = "default" in defaultImport ? defaultImport.default : defaultImport;

// esm-externals:@splunk/visualization-context/Provider
var Provider_exports = {};
__export(Provider_exports, {
  default: () => Provider_default
});
var defaultImport2 = __toESM(require("@splunk/visualization-context/Provider"));
__reExport(Provider_exports, require("@splunk/visualization-context/Provider"));
var Provider_default = "default" in defaultImport2 ? defaultImport2.default : defaultImport2;

// src/BaseRegistry.js
var BaseRegistry = class _BaseRegistry {
  constructor() {
    this.cachedProviders = {};
    this.defaultProvider = null;
  }
  static create() {
    return new _BaseRegistry();
  }
  /**
   * check whether user defined provider extends the base provider
   * @param {Provider} provider
   * @returns {Boolean}
   */
  isValidProvider(provider) {
    if (provider instanceof Provider_default) {
      return true;
    }
    throw new Error((0, i18n_exports._)("Invalid Provider"));
  }
  /**
   * add default provider to cache
   * @param {Provider} provider
   */
  addDefaultProvider(provider) {
    if (this.isValidProvider(provider)) {
      this.addProvider(provider);
      this.defaultProvider = provider;
    }
  }
  /**
   * add provider to cache
   * @param {Provider} provider
   */
  addProvider(provider) {
    if (this.isValidProvider(provider)) {
      if (this.cachedProviders[provider.getType()]) {
        console.warn(`The provider "${provider.getType()}" existed already.`);
      }
      this.cachedProviders[provider.getType()] = provider;
    }
  }
  /**
   * delete provider
   * @param {String} providerType
   */
  deleteProvider(providerType) {
    const cachedProvider = this.cachedProviders[providerType];
    if (!cachedProvider) {
      console.warn(`The provider "${providerType}" does not existed`);
      return;
    }
    if (this.defaultProvider && providerType === this.defaultProvider.getType()) {
      this.defaultProvider = null;
    }
    delete this.cachedProviders[providerType];
  }
  /**
   * find provider from cache
   * @param {String} type
   * @returns {Provider} provider
   */
  findProvider(type) {
    const provider = this.cachedProviders[type];
    if (provider) {
      return provider;
    }
    throw new Error((0, i18n_exports._)("Provider not found"));
  }
  /**
   * list all providers
   * @return {Object} provider = { type: 'local', isDefault: true }
   */
  listProviders() {
    return Object.keys(this.cachedProviders).map((type) => ({
      type,
      isDefault: type === this.defaultProvider.getType()
    }));
  }
  /**
   * check whether src is a http url or a url used for icon/image-registry (e.g. 'kvstore://image123')
   */
  isResourceURL(url) {
    if (!url || url.indexOf("://") < 0) {
      return false;
    }
    const { type } = this.parseURL(url);
    return type !== "http" && type !== "https";
  }
  /**
   * parse provider type and id from the url
   * @params {String} url     'kvstore://image123'
   * @returns {Object} obj
   * @returns {String} obj.type   'kvstore'
   * @returns {String} obj.id     'image123'
   */
  parseURL(url) {
    if (url) {
      const [type, id] = url.split("://");
      if (type && id) {
        return { type, id };
      }
    }
    throw new Error((0, i18n_exports._)("Invalid url"));
  }
  /**
   * upload
   * @param {DataURI} dataURI
   * @param {Object} metaData
   * @param {String} providerType      e.g. 'kvstore' or 'sui' or 'local'
   * @returns {String} id               e.g. 'kvstore://image123'
   */
  upload(dataURI, metaData, providerType) {
    return __async(this, null, function* () {
      if (providerType == null && this.defaultProvider == null) {
        throw new Error((0, i18n_exports._)("No valid provider was found. Please add a provider"));
      }
      const provider = providerType ? this.findProvider(providerType) : this.defaultProvider;
      const id = yield provider.upload(dataURI, metaData);
      return `${provider.getType()}://${id}`;
    });
  }
  /**
   * get object by url
   * @param {String} url     'kvstore://image123'
   * @returns {Object} obj    e.g. an image object or icon
   */
  getByURL(url) {
    return __async(this, null, function* () {
      const { type, id } = this.parseURL(url);
      const provider = this.findProvider(type);
      const obj = yield provider.getById(id);
      return obj;
    });
  }
  /**
   * delete object by url
   * @param {String} url     'kvstore://image123' || 'sui://icon123'
   * @returns {Object} obj    e.g. an image object or icon
   */
  deleteByURL(url) {
    return __async(this, null, function* () {
      const { type, id } = this.parseURL(url);
      const provider = this.findProvider(type);
      const obj = yield provider.deleteById(id);
      return obj;
    });
  }
  /**
   * list all object ids
   * @params {String} providerType       e.g. 'kvstore' or 'sui'
   * @params {Object} options
   * @returns {Array} ids
   */
  listIds(_0) {
    return __async(this, arguments, function* (providerType, options = { offset: 0, count: 100 }) {
      const provider = this.findProvider(providerType);
      const ids = yield provider.listIds(options);
      return ids;
    });
  }
};
var BaseRegistry_default = BaseRegistry;

// src/ImageRegistry.js
var _ImageRegistry = class _ImageRegistry extends BaseRegistry_default {
  static create() {
    return new _ImageRegistry();
  }
  /**
   * check if the the media type from the dataURI belongs to the supported types
   * @param {DataURI} dataURI
   * @returns {Boolean}
   */
  isValidMediaType(dataURI, validMediaType = _ImageRegistry.VALID_MEDIA_TYPES) {
    return validMediaType.some((type) => {
      const regex = `^data:image\\/${type === "svg" ? `${type}\\+xml` : type};base64,.*$`;
      return new RegExp(regex).test(dataURI);
    });
  }
  /**
   * upload image if the image format is supported
   * @param {DataURI} dataURI
   * @param {Object} metaData
   * @param {String} providerType   e.g. 'kvstore', or 'sui'
   * @returns {String} id           e.g. 'kvstore://image123'
   */
  upload(dataURI, metaData, providerType, validMediaType = _ImageRegistry.VALID_MEDIA_TYPES) {
    if (!this.isValidMediaType(dataURI, validMediaType)) {
      throw new Error(
        (0, i18n_exports._)(
          `The format of the uploaded file is not supported. Please upload ${validMediaType.join(
            ", "
          )}.`
        )
      );
    }
    return super.upload(dataURI, metaData, providerType);
  }
};
__publicField(_ImageRegistry, "VALID_MEDIA_TYPES", ["png", "jpeg", "svg", "gif", "webp"]);
var ImageRegistry = _ImageRegistry;
var ImageRegistry_default = ImageRegistry;
//# sourceMappingURL=ImageRegistry.js.map
